
import time,os

import pygame
from pygame.locals import *
if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

TRANSPARENT_COLOR=(17,136,17)

from common import *
import factory
import map

class Game:

  def __init__(s):
    # sdl init
    s.SDL_wsize=pygame.Rect( 0,0,13*s.tilesize(),13*s.tilesize() )
    s.SDL_screen = pygame.display.set_mode( s.SDL_wsize[2:],DOUBLEBUF )
    pygame.mouse.set_visible(1)
    s.SDL_background = pygame.Surface(s.SDL_screen.get_size()).convert()
    s.SDL_clock=pygame.time.Clock()
    s.SDL_fonts={}
    s.SDL_bgcolor=(0,0,255)
    s.keys=[]
    s.offset=[0,0]
    s.images={}
    s.isize=[0,0]
    # SDL groups
    s.SDLGR_all=pygame.sprite.RenderPlain()
    s.SDLGR_map=pygame.sprite.RenderPlain()
    s.SDLGR_map_on_display=pygame.sprite.RenderPlain()
    # game stuff
    s.fact=factory.ThingFactory(s)
    s.map=None

  def tilesize(s):
    return 48

  def image_cache(s,dir,fname,colorkey=TRANSPARENT_COLOR):
    fpath=os.path.join(dir,fname)
    image = pygame.image.load(fpath).convert()
    if colorkey is not None:
      if colorkey is -1:
        colorkey = image.get_at( AT_0_0 )
      image.set_colorkey(colorkey, RLEACCEL)
#    print '>>',image
#    print '>>',image.get_rect()
#    print '>>',type(image)
    s.images[ fname ]= image
#    name=os.path.splitext( os.path.split(fpath)[-1] )[0]
#    s.images[ name ]= image

  def map_load(s,name):
    s.map=map.Map(s,name)
    return s.map
  def fact_has_type(s,type):
    return s.fact.has_type(type)
  def fact_create_tile(s,pos,codes):
    tile=s.fact.create_tile(pos,codes)
    s.SDLGR_all.add( tile )
    s.SDLGR_map.add( tile )
    return tile
  def fact_create_objects(s,pos,objects):
    obj=s.fact.create_objects(pos,objects)
    s.SDLGR_all.add( obj )
    s.SDLGR_objects.add( obj )
    return obj

  def update_brains(s):
#    s.GR_brains_event.update( UPDATE_PROCESS, time.time(), )
    pass

  def SDL_event(s,event):
    if event.type == QUIT:
      if DEBUG: print 'SDL_game: Window manager said QUIT!!!'
      s.active=0
    elif event.type == KEYDOWN:
      k=event.key
      if k not in s.keys: s.keys.append( k )
    elif event.type == KEYUP:
      k=event.key
      if k in s.keys: s.keys.remove( k )
    elif event.type == MOUSEBUTTONDOWN:
      print "RAT CLICK!!!",event
    elif event.type == MOUSEBUTTONUP:
      pass
    elif event.type == MOUSEMOTION:
      pass
    else:
      print 'SDL_event: unhandled event',event

  def SDL_offset_delta(s,dx,dy):
    x,y=s.offset
    dpos=( dx*s.tilesize(), dy*s.tilesize() )
    s.SDLGR_map_on_display.empty()
    s.SDLGR_all.update( UPDATE_THING__OFFSET_DELTA,
                              dpos,
                              s.SDLGR_map_on_display,
                              s.map.SDL_map_rect,
                              )
    print len(s.SDLGR_map_on_display)
    s.map.redraw=1
    s.offset=[x+dx,y+dx]

  def SDL_keys(s):
    if K_ESCAPE in s.keys:
      s.active=0
    if K_LEFT in s.keys:
      s.SDL_offset_delta(.1,0)
    if K_RIGHT in s.keys:
      s.SDL_offset_delta(-.1,0)
    if K_UP in s.keys:
      s.SDL_offset_delta(0,.1)
    if K_DOWN in s.keys:
      s.SDL_offset_delta(0,-.1)


  def SDL_refresh(s):
    s.map.SDL_refresh()
    s.SDL_background.fill( s.SDL_bgcolor )
    s.SDL_screen.blit(s.map.SDL_map, (0, 0))
    ox,oy=s.offset
    x1=ox+0
    y1=oy+0
    x2=ox+s.SDL_wsize[0]
    y2=ox+s.SDL_wsize[1]
    pygame.display.flip()


  def SDL_loop(s):
    s.active=1
    t1=time.time()
    count=last=0
    while s.active:
      s.SDL_clock.tick(100)

      events=pygame.event.get()
      for event in events:
        s.SDL_event(event)
      s.SDL_keys()

      # do brains
      s.update_brains()
      # check map update?
      s.SDL_refresh()

      count=count+1
      t2=time.time()
      if int(t1)!=int(t2):
        t1=t2
        print 'fps:',count-last
        last=count

